package model

import "live-service/app/user_center/rpc/pb"

var rankAndScoreMap map[pb.RankType]ScoreType
var scoreAndRankMap map[ScoreType]pb.RankType

func ScoreTypeByRankType(rankType pb.RankType) ScoreType {
	if scoreType, ok := rankAndScoreMap[rankType]; ok {
		return scoreType
	}
	return ScoreTypeDamage
}

func RankTypeByScoreType(scoreType ScoreType) pb.RankType {
	if rankType, ok := scoreAndRankMap[scoreType]; ok {
		return rankType
	}
	return pb.RankType_Damage
}

func AllScoreType() []ScoreType {
	result := make([]ScoreType, 0, len(rankAndScoreMap))
	for _, scoreType := range rankAndScoreMap {
		result = append(result, scoreType)
	}
	return result
}

func addRankScore(rankType pb.RankType, scoreType ScoreType) {
	rankAndScoreMap[rankType] = scoreType
	scoreAndRankMap[scoreType] = rankType
}

func init() {
	rankAndScoreMap = make(map[pb.RankType]ScoreType)
	scoreAndRankMap = make(map[ScoreType]pb.RankType)

	addRankScore(pb.RankType_Damage, ScoreTypeDamage)
	addRankScore(pb.RankType_DeDamage, ScoreTypeDeDamage)
	addRankScore(pb.RankType_General, ScoreTypeGeneral)
	addRankScore(pb.RankType_DeGeneral, ScoreTypeDeGeneral)
	addRankScore(pb.RankType_KillUnit, ScoreTypeKillUnit)
	addRankScore(pb.RankType_DeKillUnit, ScoreTypeDeKillUnit)
	addRankScore(pb.RankType_KillPlayer, ScoreTypeKillPlayer)
	addRankScore(pb.RankType_DeKillPlayer, ScoreTypeDeKillPlayer)
	addRankScore(pb.RankType_Win, ScoreTypeWin)
	addRankScore(pb.RankType_Lost, ScoreTypeLost)
	addRankScore(pb.RankType_FirstBlood, ScoreTypeFirstBlood)
	addRankScore(pb.RankType_DeFirstBlood, ScoreTypeDeFirstBlood)
}
