package gift

import (
	"context"
	"github.com/pkg/errors"
	"github.com/zeromicro/go-zero/core/logx"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	"live-service/common/nerr"
	"live-service/common/timex"
)

type UserBuyNobilityLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewUserBuyNobilityLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UserBuyNobilityLogic {
	return &UserBuyNobilityLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// UserBuyNobility 用户购买贵族
// 1. 记录送礼 + 新增积分
// 2. 修改贵族
func (l *UserBuyNobilityLogic) UserBuyNobility(in *pb.UserBuyNobilityReq) (*pb.UserBuyNobilityResp, error) {
	// 记录送礼信息
	sendGiftLogic := NewUserSendGiftLogic(l.ctx, l.svcCtx)
	integralChanged, err := sendGiftLogic.UserSendGift(&pb.UserSendGiftReq{
		Platform: in.Platform,
		UserId:   in.UserId,
		PUid:     in.PUid,
		RoomId:   in.RoomId,
		GiftId:   in.GiftId,
		GiftName: in.GiftName,
		Num:      in.Num,
		Price:    in.Price,
		IsPaid:   true,
		BattleId: in.BattleId,
	})
	if err != nil {
		return nil, err
	}
	// 修改贵族
	err = l.svcCtx.UserNobilityModel.RecordNobility(l.ctx, &model.UserNobility{
		UserId:        in.UserId,
		NobilityLevel: in.Level,
		Forever:       false,
		StartTime:     timex.TimestampToTime(in.StartTime),
		EndTime:       timex.TimestampToTime(in.EndTime),
	})
	if err != nil {
		return nil, errors.Wrapf(nerr.NewError(nerr.GiftNobilityUpdateErr, "更新贵族信息失败"), "更新贵族信息失败 err:%v", err)
	}

	return &pb.UserBuyNobilityResp{
		Integral: &pb.ChangeIntegralResp{
			UserId:   integralChanged.Integral.UserId,
			Change:   integralChanged.Integral.Change,
			Integral: integralChanged.Integral.Integral,
		},
	}, nil
}
