package integral

import (
	"context"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	"live-service/common/nerr"

	"github.com/zeromicro/go-zero/core/logx"
)

type TransferUserIntegralLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewTransferUserIntegralLogic(ctx context.Context, svcCtx *svc.ServiceContext) *TransferUserIntegralLogic {
	return &TransferUserIntegralLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// TransferUserIntegral 转移积分
func (l *TransferUserIntegralLogic) TransferUserIntegral(in *pb.TransferUserIntegralReq) (*pb.TransferUserIntegralResp, error) {
	transResp := l.svcCtx.IntegralManager.TransferIntegral(l.ctx, in)
	if transResp.Code == int32(nerr.OK) {
		// 获取 username
		if tmp, err := l.svcCtx.UserPlatformModel.FindDisplayOneByUserId(l.ctx, transResp.UserId); err == nil {
			transResp.Uname = tmp.PUname
			transResp.Avatar = tmp.PAvatar
		}
		if tmp, err := l.svcCtx.UserPlatformModel.FindDisplayOneByUserId(l.ctx, transResp.TargetUserId); err == nil {
			transResp.TargetUname = tmp.PUname
			transResp.TargetAvatar = tmp.PAvatar
		}
	}
	return transResp, nil
}
